@@ -1,4 +1,4 @@
-﻿/*  =========================================================================
+/*  =========================================================================
     zhash - simple generic hash container
 
     Copyright (c) the Contributors as noted in the AUTHORS file.
@@ -22,7 +22,7 @@
 @end
 */
 
-#include "../include/czmq.h"
+#include "czmq_classes.h"
 
 //  Hash table performance parameters
 
@@ -73,12 +73,10 @@ zhash_t *
 zhash_new (void)
 {
     zhash_t *self = (zhash_t *) zmalloc (sizeof (zhash_t));
-    if (self) {
-        self->limit = INITIAL_SIZE;
-        self->items = (item_t **) zmalloc (sizeof (item_t *) * self->limit);
-        if (!self->items)
-            zhash_destroy (&self);
-    }
+    assert (self);
+    self->limit = INITIAL_SIZE;
+    self->items = (item_t **) zmalloc (sizeof (item_t *) * self->limit);
+    assert (self->items);
     return self;
 }
 
@@ -186,7 +184,6 @@ zhash_insert (zhash_t *self, const char *key, void *value)
         self->items = new_items;
         self->limit = new_limit;
     }
-
     return s_item_insert (self, key, value)? 0: -1;
 }
 
@@ -220,11 +217,13 @@ s_item_insert (zhash_t *self, const char *key, void *value)
     item_t *item = s_item_lookup (self, key);
     if (item == NULL) {
         item = (item_t *) zmalloc (sizeof (item_t));
-        if (!item)
-            return NULL;
+        assert (item);
+
         //  If necessary, take duplicate of item (string) value
-        if (self->autofree)
+        if (self->autofree) {
             value = strdup ((char *) value);
+            assert (value);
+        }
         item->value = value;
         item->key = strdup (key);
         item->index = self->cached_index;
@@ -280,8 +279,10 @@ zhash_update (zhash_t *self, const char *key, void *value)
             free (item->value);
 
         //  If necessary, take duplicate of item (string) value
-        if (self->autofree)
+        if (self->autofree) {
             value = strdup ((char *) value);
+            assert (value);
+        }
         item->value = value;
     }
     else
@@ -335,6 +336,7 @@ zhash_rename (zhash_t *self, const char *old_key, const char *new_key)
         s_item_destroy (self, old_item, false);
         free (old_item->key);
         old_item->key = strdup (new_key);
+        assert (old_item->key);
         old_item->index = self->cached_index;
         old_item->next = self->items [self->cached_index];
         self->items [self->cached_index] = old_item;
@@ -354,7 +356,7 @@ zhash_rename (zhash_t *self, const char *old_key, const char *new_key)
 //  Returns the item, or NULL if there is no such item.
 
 void *
-zhash_freefn (zhash_t *self, const char *key, zhash_free_fn *free_fn)
+zhash_freefn (zhash_t *self, const char *key, zhash_free_fn free_fn)
 {
     assert (self);
     assert (key);
@@ -515,7 +517,7 @@ zhash_comment (zhash_t *self, const char *format, ...)
         va_end (argptr);
         if (string)
             zlist_append (self->comments, string);
-        free (string);
+        zstr_free (&string);
     }
     else
         zlist_destroy (&self->comments);
@@ -575,40 +577,33 @@ zhash_load (zhash_t *self, const char *filename)
 
     //  Take copy of filename in case self->filename is same string.
     char *filename_copy = strdup (filename);
-    if (filename_copy) {
-        free (self->filename);
-        self->filename = filename_copy;
-        self->modified = zsys_file_modified (self->filename);
-        FILE *handle = fopen (self->filename, "r");
-        if (handle) {
-            char *buffer = (char *) zmalloc (1024);
-            if (buffer) {
-                while (fgets (buffer, 1024, handle)) {
-                    //  Skip lines starting with "#" or that do not look like
-                    //  name=value data.
-                    char *equals = strchr (buffer, '=');
-                    if (buffer [0] == '#' || equals == buffer || !equals)
-                        continue;
+    assert (filename_copy);
+    free (self->filename);
+    self->filename = filename_copy;
+    self->modified = zsys_file_modified (self->filename);
 
-                    //  Buffer may end in newline, which we don't want
-                    if (buffer [strlen (buffer) - 1] == '\n')
-                        buffer [strlen (buffer) - 1] = 0;
-                    *equals++ = 0;
-                    zhash_update (self, buffer, equals);
-                }
-                free (buffer);
-            }
-            else {
-                fclose (handle);
-                return -1; // Out of memory
-            }
-            fclose (handle);
+    FILE *handle = fopen (self->filename, "r");
+    if (handle) {
+        char *buffer = (char *) zmalloc (1024);
+        assert (buffer);
+        while (fgets (buffer, 1024, handle)) {
+            //  Skip lines starting with "#" or that do not look like
+            //  name=value data.
+            char *equals = strchr (buffer, '=');
+            if (buffer [0] == '#' || equals == buffer || !equals)
+                continue;
+
+            //  Buffer may end in newline, which we don't want
+            if (buffer [strlen (buffer) - 1] == '\n')
+                buffer [strlen (buffer) - 1] = 0;
+            *equals++ = 0;
+            zhash_update (self, buffer, equals);
         }
-        else
-            return -1; // Failed to open file for reading
+        free (buffer);
+        fclose (handle);
     }
     else
-        return -1; // Out of memory
+        return -1; // Failed to open file for reading
 
     return 0;
 }
@@ -781,39 +776,12 @@ zhash_autofree (zhash_t *self)
 }
 
 
-//  --------------------------------------------------------------------------
-//  DEPRECATED as clumsy -- use zhash_first/_next instead
-//  Apply function to each item in the hash table. Items are iterated in no
-//  defined order.  Stops if callback function returns non-zero and returns
-//  final return code from callback function (zero = success).
-
-int
-zhash_foreach (zhash_t *self, zhash_foreach_fn *callback, void *argument)
-{
-    assert (self);
-
-    uint index;
-    for (index = 0; index < self->limit; index++) {
-        item_t *item = self->items [index];
-        while (item) {
-            //  Invoke callback, passing item properties and argument
-            item_t *next = item->next;
-            int rc = callback ((const char *) item->key, item->value, argument);
-            if (rc)
-                return rc;      //  End if non-zero return code
-            item = next;
-        }
-    }
-    return 0;
-}
-
-
 //  --------------------------------------------------------------------------
 //  Runs selftest of class
 //
 
 void
-zhash_test (int verbose)
+zhash_test (bool verbose)
 {
     printf (" * zhash: ");
 
